import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 11.10.16
 */
public class TestGen {
    static class Test {
        int n;
        Query[] queries;
        String descr;

        public Test(int n, Query[] queries, String descr) {
            this.n = n;
            this.queries = queries;
            this.descr = descr;
        }

        static class Query {
            int tp;
            int l, r;
            String s;

            int i;

            public Query(int i, int l, int r) {
                this.tp = 2;
                this.i = i;
                this.l = l;
                this.r = r;
            }

            public Query(int l, int r, String s) {
                this.tp = 1;
                this.l = l;
                this.r = r;
                this.s = s;
            }
        }


        void inc() {
            for (Query query : queries) {
                query.l++;
                query.r++;
                query.i++;
            }
        }

        public static TestBuilder builder() {
            return new TestBuilder();
        }

        public static class TestBuilder {
            private int n;
            private String descr;
            List<Query> q = new ArrayList<>();
            Fenwick fenwick;

            public TestBuilder n(int n) {
                fenwick = new Fenwick(n);
                this.n = n;
                return this;
            }

            public TestBuilder descr(String descr) {
                this.descr = descr;
                return this;
            }

            public TestBuilder add(Query query) {
                q.add(query);
                if (query.tp == 1) {
                    fenwick.add(query.l, query.r, query.s.length());
                }
                return this;
            }

            public TestBuilder add(int i, int l, int r) {
                q.add(new Query(i, l, r));
                return this;
            }

            public TestBuilder add(int l, int r, String s) {
                q.add(new Query(l, r, s));
                fenwick.add(l, r, s.length());
                return this;
            }

            public int getLength(int id) {
                return fenwick.get(id);
            }

            public Test build() {
                Query[] queries = new Query[q.size()];
                q.toArray(queries);
                return new Test(this.n, queries, descr);
            }
        }
    }

    private final boolean FOR_POLYGON = true;

    public static final int MAX_N = 2_00_000;
    public static final int MAX_SUM = 1_000_000;

    int testCount;
    static Random random = new Random(0xF00C);

    private String folder;
    boolean noModifyTest = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i]);
            else writer.print(arr[i] + " ");
    }

    public void print(Test test) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        if (!noModifyTest) {
            test.inc();
        }

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + test.descr);
        Test.Query[] queries = test.queries;
        out.println(test.n + " " + queries.length);
        for (Test.Query query : queries) {
            if (query.tp == 1) {
                out.println(query.tp + " " + query.l + " " + query.r + " " + query.s);
            } else {
                out.println(query.tp + " " + query.i + " " + query.l + " " + query.r);
            }
        }
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        noModifyTest = true;

        System.out.println("==== Generating samples ===");
        print(new Test(4, new Test.Query[] {
                new Test.Query(2, 2, "two"),
                new Test.Query(2, 3, "aa"),
                new Test.Query(3, 1, 1),
                new Test.Query(2, 1, 4)},
                "sample1"));

        print(new Test(3, new Test.Query[] {
                new Test.Query(1, 2, "ab"),
                new Test.Query(2, 3, "cd"),
                new Test.Query(2, 2, 4),
                new Test.Query(3, 1, 2),
                new Test.Query(1, 3, "xyzu"),
                new Test.Query(1, 2, 5)},
                "sample2"));
        noModifyTest = false;
    }

    void genHandleTests() throws IOException {
        noModifyTest = true;
        System.out.println("==== Generating handle tests ====");
        Test.TestBuilder builder = Test.builder();
        builder.n(8);
        builder.add(2, 5, "abc");
        builder.add(3, 6, "xyzs");
        builder.add(4, 5, 6);
        builder.add(2, 7, "lool");
        builder.add(1, 3, "append");
        builder.add(2, 3, 9);
        builder.descr("Handle test1");
        print(builder.build());
        noModifyTest = false;
    }

    void genTests() throws IOException {
        System.out.println("==== Generating tests ====");
        print(bigAppend(MAX_N, 1, MAX_SUM, 1));
        print(bigAppend(MAX_N, 2, MAX_SUM, 1));
        print(bigAppend(MAX_N, 1, MAX_SUM, 10));
        print(bigAppend(MAX_N, 5, MAX_SUM, 10));
        print(bigAppend(MAX_N, 3, MAX_SUM, 1000));
        print(bigAppend(MAX_N, 5, MAX_SUM, 1000));
        print(bigAppend(MAX_N, 4, MAX_SUM, 100000));
        print(bigAppend(MAX_N, 3, MAX_SUM, MAX_N));

        print(genInfrequent(MAX_N, 0.1));
        print(genInfrequent(MAX_N, 0.01));
        print(genInfrequent(MAX_N, 0.001));
        print(genInfrequent(MAX_N, 0.0001));
        print(genInfrequent(MAX_N, 0.00008));

        int[] appends = new int[]{1, 10, 30, 100, 2000, 100000, MAX_N - START_APPENDS};
        for (int append : appends) {
            print(genRandom(MAX_N, append, MAX_SUM));
        }

        int[] mxSub = new int[]  {1,  2,  3,  10,  20,   100,   1000};
        for (int i = 0; i < appends.length; ++i) {
            print(genRandom(MAX_N, appends[i], mxSub[i]));
        }

        print(gen2N_1Appends(MAX_N, MAX_N));
    }

    private Test gen2N_1Appends(int n, int m) {
        int ln = MAX_SUM / (m - 1);
        Test.TestBuilder builder = Test.builder();
        builder.n(n);
        for (int i = 1; i < m; ++i) {
            builder.add(1, n - 2, randString(ln));
        }
        builder.add(randInt(1, n - 2), 1, ln * (m - 1) - 1);
        builder.descr("append to [2..n-1]");
        return builder.build();
    }

    private Test genInfrequent(int n, double prob) {
        int invp = (int)(1 / prob);
        Test.TestBuilder builder = Test.builder();
        builder.n(n);
        int mxLen = MAX_SUM / (MAX_N / 2);
        int appendLength = 0;
        int firstAppends = 0;
        for (int i = 1; i <= MAX_N / 2; ++i) {
            int len = randInt(mxLen);
            String s = randString(len);
            appendLength += len;
            int l, r;
            if (randInt(1, invp) == 1) {
                firstAppends++;
                l = 0;
                r = randInt(n / 2 - 1, n / 2 + 1000);
            } else {
                l = randInt(n / 2 - 1000, n - 1);
                r = n - 1;
            }
            builder.add(l, r, s);
        }
        List<Test.Query> queries = new ArrayList<>();
        int smLen = (int)(MAX_SUM * 0.2);
        int secondHalfRequests = randInt(1, 10);
        int[] lens = genPartition(smLen, secondHalfRequests);

        for (int i = 0; i < secondHalfRequests; ++i) {
            int id;
            while (true) {
                id = randInt(n / 2, n - 1);
                if (builder.getLength(id) >= lens[i]) {
                    break;
                }
            }
            int leng = builder.getLength(id);
            int l = randInt(0, leng - lens[i]);
            int r = l + lens[i] - 1;
            queries.add(new Test.Query(id, l, r));
        }

        int sm = MAX_SUM - smLen;
        for (int i = 1; i <= MAX_N / 2 - secondHalfRequests && sm > 0; ++i) {
            int id = randInt(0, n / 2 - 1);
            int leng = builder.getLength(id);
            int len = randInt(1, Math.min(leng, Math.min(sm, 100)));
            int l = randInt(0, leng - len);
            queries.add(new Test.Query(id, l, l + len - 1));
            sm -= len;
        }
        shuffle(queries);
        for (Test.Query q: queries) {
            builder.add(q);
        }
        builder.descr(String.format("Infrequent test: n = %d, append length = %d, requestLength = %d, "  +
                "m = %d,  first half requests = %d, first half appends = %d from %d",
                n, appendLength, MAX_SUM - sm,
                queries.size(), queries.size() - secondHalfRequests, firstAppends, MAX_N / 2));
        return builder.build();
    }

    private Test bigAppend(int n, int strNums, int maxSum, int requests) {//TODO write here
        Test.TestBuilder builder = Test.builder();
        builder.n(n);
        int[] lengs = genPartition(MAX_SUM, strNums);
        for (int i = 0; i < strNums; ++i) {
            builder.add(0, n - 1, randString(lengs[i]));
        }

        requests = Math.min(MAX_N - strNums, requests);
        int[] lengths = genPartition(maxSum, requests);
        for (int i = 0; i < requests; ++i) {
            int id = randInt(0, n - 1);
            int l = randInt(0, MAX_SUM - lengths[i]);
            builder.add(id, l, l + lengths[i] - 1);
        }
        builder.descr(String.format("Big Append: n = %d, maxSum = %d, strNums = %d, requests = %d", n, maxSum, strNums, requests));
        return builder.build();
    }

    public static final int START_APPENDS = 50;

    public String randString(int len) {
        StringBuilder s = new StringBuilder();
        for (int i = 1; i <= len; ++i) {
            char rnd = (char)(randInt(26) - 1 + 'a');
            s.append(rnd);
        }
        return s.toString();
    }

    public Test.Query randQuery1(int n, int len) {
        int l = randInt(0, n - 1);
        int r = randInt(0, n - 1);
        if (l > r) {
            int t = l;
            l = r;
            r = t;
        }
        return new Test.Query(l, r, randString(len));
    }

    private Test genRandom(int n, int appends, int maxLenSubstr) {
        Test.TestBuilder builder = Test.builder();
        builder.n(n);
        int rem = MAX_SUM - START_APPENDS;
        int[] lengths = genPartition(rem, appends);
        for (int i = 1; i <= START_APPENDS; ++i) {
            builder.add(randQuery1(n, 1));
        }
        int i = START_APPENDS;
        int j = 0;
        int reqCnt = 0;
        int smAppend = START_APPENDS, smReq = 0;
        for (; i < MAX_N && (j < lengths.length || smReq < MAX_SUM); ++i)
            if (smReq == MAX_SUM || j < lengths.length && randInt(2) == 1) {
                smAppend += lengths[j];
                builder.add(randQuery1(n, lengths[j++]));
            } else {
                List<Integer> indexes = new ArrayList<>();
                while (indexes.size() < 10) {
                    int id = randInt(0, n - 1);
                    if (builder.getLength(id) > 0) {
                        indexes.add(id);
                    }
                }
                int index = indexes.get(randInt(0, indexes.size() - 1));
                int length = builder.getLength(index);
                int l = randInt(0, length - 1);
                int lenSubstr = randInt(Math.min(maxLenSubstr, Math.min(MAX_SUM - smReq, length - l)));
                smReq += lenSubstr;
                builder.add(index, l, l + lenSubstr - 1);
                reqCnt++;
            }
        builder.descr(String.format("Random test: n = %d, m = %d, appends = %d, requests = %d\n" +
                "Max length of substring = %d\n" +
                "Sum requests = %d\n" +
                "Average appends length = %d\n" +
                "Average request length = %d\n", n, i, j + START_APPENDS, reqCnt,
                                                 maxLenSubstr,
                                                 smReq,
                                                 smAppend / (j + START_APPENDS),
                                                 smReq / reqCnt));
        return builder.build();
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genTests();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    public static class Fenwick {
        private int[] s;
        private int n;

        public Fenwick(int n) {
            this.n = n;
            this.s = new int[n];
        }

        void add(int v, int val) {
            for (; v < n; v |= v + 1) {
                s[v] += val;
            }
        }

        int get(int v) {
            int ret = 0;
            for (; v >= 0; v = (v & (v+1)) - 1) {
                ret += s[v];
            }
            return ret;
        }

        void add(int l, int r, int val) {
            add(l, val);
            add(r + 1, -val);
        }
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genRandPerm(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) {
            p.add(i);
        }
        Collections.shuffle(p, random);
        int[] ret = new int[n];
        for (int i = 0; i < n; ++i) {
            ret[i] = p.get(i);
        }
        return ret;
    }

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    private void shuffle(int[] a, int from, int to) {
        int[] b = Arrays.copyOfRange(a, from, to);
        Integer[] bInteger = new Integer[b.length];
        for (int i = 0; i < b.length; ++i)
            bInteger[i] = b[i];

        List<Integer> arr = Arrays.asList(bInteger);
        shuffle(arr);
        for (int i = from; i < to; ++i)
            a[i] = arr.get(i - from);
    }

    private int[] toIntArray(Integer[] a) {
        int[] ret = new int[a.length];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = a[i];
        return ret;
    }


    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }
}
